﻿-- Daily Global Check - Garrison Follower Missions plugin
-- Jadya EU-Well of Eternity
local addonName, addonTable = ...

local pID
local pTable
local pFaction = UnitFactionGroup("player")
local questsdata, questsorder = {},{}
DGC_GarrMLocalizedStrings = {}

local textures = {"Interface\\Garrison\\Portraits\\FollowerPortrait_NoPortrait", -- normal
                  "Interface\\Garrison\\ShipsCargoShipMap"} -- naval
local questID_pref = "miss_"

-- mission types
local NORMAL = 1
local NAVAL  = 2

local plugintitle = "Follower Missions"
local pluginicon = "Interface\\Icons\\Achievement_General_CityAttacker2"

local function formattime(s)
 return string.format("%.2d:%.2d:%.2d", s/(60*60), s/60%60, s%60)
end

-- NORMAL and NAVAL missions counters
local counters = { {},{} }
local function isquestcompleted(questID)
 questID = questID:gsub(questID_pref, "")
 if not questID or not pTable[questID] or not pTable[questID].missions then return false end
 
 local g,r = "|cff00FF00","|cffFF0000"
 local sr = ""
 local now = time()
 for _,v in pairs(counters) do
  v[1] = 0
  v[2] = 0
  v[3] = ""
  v[4] = 999999999999999
 end
 
 -- for clarity purposes
 local READY = 1
 local MAX = 2
 local CSTRING = 3
 local MINOR = 4

 for k,v in pairs(pTable[questID].missions) do
  local t = counters[v.type]
  if v.timer < t[MINOR] then
   t[CSTRING] = now > v.timer and "" or formattime(v.timer - now)
   t[MINOR] = v.timer
  end
  if now > v.timer then t[READY] = t[READY] + 1 end
  t[MAX] = t[MAX] + 1
 end

 local k = #counters
 while k > 0 do
  local v = counters[k]
  local result = ""
  if v[READY] > 0 then
   result = "|T"..textures[k]..":12|t "..result..g..v[READY].." / "..v[MAX].."|r"
  elseif v[MAX] > 0 then
   result = "|T"..textures[k]..":12|t "..result..r..v[CSTRING]
  end
  sr = sr..result
  k = k - 1
 end
 return false, sr -- forced false to counter the "hide completed quests" option
end

local plugin_data

local function GenerateQuestsData()
 if not DailyGlobalCheck then return end
 wipe(questsdata)
 wipe(questsorder)
 
 local function gs(v)
  return v == nil and "" or v
 end

 -- plugin options
 local svar = DailyGlobalCheck_PluginData[plugintitle]
 local realm_headers = false
 if svar and svar.options then
  realm_headers = svar.options[1]
 end
 if realm_headers then
  questsorder[1] = {}
 else
  questsorder[1] = {{OVERVIEW}}
 end
 --
 
 local function foreachinorder(t,f)
  local keys = {}
  for k,_ in pairs(t) do
    keys[#keys+1] = k
  end
  table.sort(keys,function (a,b)
   return string.lower(a) < string.lower(b)
  end)
  for _,k in ipairs(keys) do
   f(k,t[k])
  end
 end

 foreachinorder(pTable, function(k,v)
  if v.show then
   local name, realm = strsplit("-", k)
   local faction_color = v.faction == "Horde" and "|cffFF2222" or v.faction == "Alliance" and "|cff00AAFF" or ""
   local questID = questID_pref..k
   questsdata[questID] = {gs(realm),faction_color..gs(name).."|r","",""}
   if realm_headers then
    local t
    for _,data in pairs(questsorder[1]) do
	 if data[1] == realm then
	  t = data
	  break
	 end
	end
	if not t then
	 table.insert(questsorder[1], {realm})
	 t = questsorder[1][table.getn(questsorder[1])]
	end
	table.insert(t, questID)
   else
 	table.insert(questsorder[1][1], questID)
   end
  end
 end)
 
 if DailyGlobalCheck.PushData then
  DailyGlobalCheck:PushData(questsdata, isquestcompleted, plugintitle)
 end
end

local function Initialize()
 if not DailyGlobalCheck then return end
 DailyGlobalCheck:LoadPlugin(plugin_data, questsdata)
end

local function checkvisibility()
 local found
 for k,v in pairs(pTable[pID].missions) do
  found = true
 end

 if found then return end
 
 -- no timers available
 pTable[pID].show = nil
 GenerateQuestsData()
end

local function removeunusedmissions()
 for mID,t in pairs(pTable[pID].missions) do
   local found
   for k,v in pairs(C_Garrison.GetInProgressMissions()) do
    if mID == v.missionID then found = true end
   end
   for k,v in pairs(C_Garrison.GetCompleteMissions()) do
    if mID == v.missionID then found = true end
   end
   if not found then
    pTable[pID].missions[mID] = nil
   end
  end
  checkvisibility()
end

local last_opt = {}
local function OnShow()
 removeunusedmissions()
 
 local svar = DailyGlobalCheck_PluginData[plugintitle]
 if not svar.options then return end
 
 local found
 for k,v in pairs(svar.options) do
  if v ~= last_opt[k] then
   found = true
  end
  last_opt[k] = v
 end

 if found then
  GenerateQuestsData()
 end
end

plugin_data = {
 ["Title"] = plugintitle,
 ["Icon"]  = pluginicon,
 ["Version"] = 1003,
 ["Order"] = questsorder,
 ["Data"] = {}, -- empty, for compatibility reasons
 ["Overrides"] = {["isquestcompleted"] = isquestcompleted},
 ["OnShow"] = OnShow,
 ["Options"] = {{"Checkbox","Split Realms",true,{}}},
 }

local initialized = false
local eventframe = CreateFrame("FRAME")
eventframe:RegisterEvent("ADDON_LOADED")
local function eventhandler(self, event, ...)
 if event == "ADDON_LOADED" and ... == addonName then
  if not DailyGlobalCheck then return end
  local pName = GetUnitName("player")
  local pRealm = GetRealmName()
  pID = pName.."-"..pRealm
  if not DGC_GarrMLocalizedStrings[pID] then DGC_GarrMLocalizedStrings[pID] = {} end
  
  pTable = DGC_GarrMLocalizedStrings

  for k,v in pairs(pTable) do
   -- give it a more appropriate name since we're upgrading it's contents
   if v.timers then
    v.missions = {}
    for missionID,t in pairs(v.timers) do
     v.missions[missionID] = {timer = t, type = NORMAL}
	end
	v.timers = nil
   end
  end
  if not pTable[pID].missions then pTable[pID].missions = {} end

  pTable[pID].faction = pFaction
  
  GenerateQuestsData()
  Initialize()
  initialized = true
  eventframe:RegisterEvent("GARRISON_MISSION_STARTED")
  eventframe:RegisterEvent("GARRISON_MISSION_COMPLETED")
 elseif event == "GARRISON_MISSION_COMPLETED" then
  local missionID = ...
  local t = time()
  if pTable[pID].missions[missionID] then
   pTable[pID].missions[missionID] = nil
  end
 removeunusedmissions()
 elseif event == "GARRISON_MISSION_STARTED" then
  local missionID = ...
  local typeAtlas
  for k,v in pairs(C_Garrison.GetInProgressMissions()) do
   if v.missionID == missionID then
    typeAtlas = v.typeAtlas
   end
  end
  local _, st, _, _, et = C_Garrison.GetMissionTimes(missionID)
  local t = time() + st + et
  pTable[pID].show = true
  local missionType = typeAtlas:find("ShipMissionIcon") and NAVAL or NORMAL
  pTable[pID].missions[missionID] = { timer = t, type = missionType }
  removeunusedmissions()
  GenerateQuestsData()
 end
end
eventframe:SetScript("OnEvent", eventhandler)